@@ -4,7 +4,7 @@
  * Copyright 1994 Alexander Jean-Claude Bottema
  * Copyright 1995 Stephen Lee
  * Copyright 1997, 1998 Aaron Culliney
- * Copyright 1998, 1999 Michael Deutschmann
+ * Copyright 1998, 1999, 2000 Michael Deutschmann
  *
  * This software package is subject to the GNU General Public License
  * version 2 or later (your choice) as published by the Free Software 
@@ -28,21 +28,15 @@
 #include <errno.h>
 
 #include "interface.h"
-#include "apple2.h"
 #include "keys.h"
 #include "disk.h"
 #include "misc.h"
 #include "video.h"
+#include "cpu.h"
+#include "prefs.h"
 
 static struct stat	statbuf;
 static int		altdrive;
-#ifdef _640x400
-static unsigned char	intermediate_font[0x800*8];
-static unsigned char	interface_font[0x800*16];
-#else
-#define		intermediate_font interface_font
-static unsigned char	interface_font[0x800*8];
-#endif
 
 /*#define undoc_supported 1*/
 /*#else*/
@@ -54,9 +48,15 @@ static unsigned char	interface_font[0x800*8];
 #define iie_supported 0
 #endif
 
+static void pad_string(char *s, char c, int len) {
+    char *p;
 
-/* from prefs.l */
-extern void pad_string(char*, char, int);
+    for (p = s; ((*p != '\0') && (p-s < len-1)); p++);
+    while (p-s < len-1) {
+	*p++ = c;
+    }
+    *p = '\0';
+}
 
 /* in keys.c */
 //extern void c_mouse_close();
@@ -70,22 +70,6 @@ extern void c_calibrate_joystick();
 extern long js_timelimit;
 #endif
 
-/* HACK around problems with redrawing screen on exit from the menus.
-   this problem happens when auxram is being used. */
-unsigned char updating_screen = 0;
-
-/* -------------------------------------------------------------------------
-    c_load_character()
-   ------------------------------------------------------------------------- */
-void c_load_character( int ascii, unsigned char bitmap[8][8] )
-{
-    int		i, j;
-
-    for (i = 0; i < 8; i++)
-	for (j = 0; j < 7; j++)
-	    intermediate_font[64 * ascii + i * 8 + j] = 
-	        (bitmap[ i ][ j ] == '*') ? COLOR_LIGHT_WHITE : 0;
-}
 
 /* -------------------------------------------------------------------------
     c_load_interface_font()
@@ -93,258 +77,30 @@ void c_load_character( int ascii, unsigned char bitmap[8][8] )
 
 void c_load_interface_font()
 {
-    int			c, i, j, v, p;
+    /* Only codes 0x20 -- 0x8A are actually used. But I feel safer 
+     * explicitly initializing all of them.
+     */
 
-    static unsigned char ul_corner[8][8] = { ".......",
-					     ".......",
-					     ".......",
-					     ".......",
-					     "...****",
-					     "...*...",
-					     "...*...",
-					     "...*..." };
+    video_loadfont_int(0x00,0x40,ucase_glyphs);
+    video_loadfont_int(0x40,0x20,ucase_glyphs);
+    video_loadfont_int(0x60,0x20,lcase_glyphs);
+    video_loadfont_int(0x80,0x40,ucase_glyphs);
+    video_loadfont_int(0xC0,0x20,ucase_glyphs);
+    video_loadfont_int(0xE0,0x20,lcase_glyphs);
 
-    static unsigned char ur_corner[8][8] = { ".......",
-					     ".......",
-					     ".......",
-					     ".......",
-					     "****...",
-					     "...*...",
-					     "...*...",
-					     "...*..." };
-
-    static unsigned char dl_corner[8][8] = { "...*...",
-					     "...*...",
-					     "...*...",
-					     "...*...",
-					     "...****",
-					     ".......",
-					     ".......",
-					     "......." };
-
-    static unsigned char dr_corner[8][8] = { "...*...",
-					     "...*...",
-					     "...*...",
-					     "...*...",
-					     "****...",
-					     ".......",
-					     ".......",
-					     "......." };
-
-    static unsigned char    v_line[8][8] = { "...*...",
-					     "...*...",
-					     "...*...",
-					     "...*...",
-					     "...*...",
-					     "...*...",
-					     "...*...",
-					     "...*..." };
-
-    static unsigned char    h_line[8][8] = { ".......",
-					     ".......",
-					     ".......",
-					     ".......",
-					     "*******",
-					     ".......",
-					     ".......",
-					     "......." };
-
-    static unsigned char   tl_junc[8][8] = { "...*...",
-					     "...*...",
-					     "...*...",
-					     "...*...",
-					     "...****",
-					     "...*...",
-					     "...*...",
-					     "...*..." };
-
-    static unsigned char   tr_junc[8][8] = { "...*...",
-					     "...*...",
-					     "...*...",
-					     "...*...",
-					     "****...",
-					     "...*...",
-					     "...*...",
-					     "...*..." };
-
-    static unsigned char   tu_junc[8][8] = { ".......",
-					     ".......",
-					     ".......",
-					     ".......",
-					     "*******",
-					     "...*...",
-					     "...*...",
-					     "...*..." };
-
-    static unsigned char   td_junc[8][8] = { "...*...",
-					     "...*...",
-					     "...*...",
-					     "...*...",
-					     "*******",
-					     ".......",
-					     ".......",
-					     "......." };
-
-    static unsigned char   x_junc[8][8] =  { "...*...",
-					     "...*...",
-					     "...*...",
-					     "...*...",
-					     "*******",
-					     "...*...",
-					     "...*...",
-					     "...*..." };
-
-    for (p = 0, c = 0; c < 256; c++)
-	for (i = c * 8; i < c * 8 + 8; i++, p++)
-	    for (v = char_rom[ i ], j = 0; j < 7; j++, v >>= 1, p++)
-	        intermediate_font[ p ] = (v & 1) ? COLOR_LIGHT_WHITE : 0;
-
-    for (c = 0; c < 96; c++)
-	for (i = 0; i < 64; i++)
-	    intermediate_font[ (c + 32) * 64 + i ] =
-	    intermediate_font[ (c + 160) * 64 + i ];
-
-    c_load_character( 0x80, ul_corner );
-    c_load_character( 0x81, ur_corner );
-    c_load_character( 0x82, dl_corner );
-    c_load_character( 0x83, dr_corner );
-    c_load_character( 0x84, v_line );
-    c_load_character( 0x85, h_line );
-    c_load_character( 0x86, tl_junc );
-    c_load_character( 0x87, tr_junc );
-    c_load_character( 0x88, tu_junc );
-    c_load_character( 0x89, td_junc );
-    c_load_character( 0x8A, x_junc );
-
-    for (c = 32; c < 128; c++)
-	for (i = 0; i < 64; i++)
-	    intermediate_font[ (c + 128) * 64 + i ] =
-	    (intermediate_font[ c * 64 + i ] == 0) ? COLOR_LIGHT_WHITE : 0;
-
-#ifdef _640x400
-    for (i=0, c=0; i<32768; i++, c++) {
-	interface_font[i] = intermediate_font[c];
-	interface_font[++i] = intermediate_font[c];
-    }
-#endif
-}
-
-/* -------------------------------------------------------------------------
-    c_interface_textcolor()
-   ------------------------------------------------------------------------- */
-
-void c_interface_textcolor( short foreground, short background )
-{
-    static short	prev_foreground = COLOR_LIGHT_WHITE;
-    static short	prev_background = 0;
-
-    int			i;
-
-    for (i=0;
-#ifdef _640x400
-	    i<32768;
-#else
-	    i<16384;
-#endif
-	    i++)
-    {
-	if (interface_font[ i ] == prev_foreground)
-	    interface_font[ i ] = foreground;
-	else
-	if (interface_font[ i ] == prev_background)
-	    interface_font[ i ] = background;
-    }
-
-    prev_background = background;
-    prev_foreground = foreground;
-}
-
-/* -------------------------------------------------------------------------
-    c_interface_print_char40()
-   ------------------------------------------------------------------------- */
-
-
-static void c_interface_print_char40_line(
-	unsigned char **d, unsigned char **s)
-{
-#ifdef _640x400
-    *((unsigned int *)(*d)) = *((unsigned int *)(*s));/*32bits*/
-    *d += 4, *s += 4;
-    *((unsigned int *)(*d)) = *((unsigned int *)(*s));/*32bits*/
-    *d += 4, *s += 4;
-    *((unsigned int *)(*d)) = *((unsigned int *)(*s));/*32bits*/
-    *d += 4, *s += 4;
-    *((unsigned short *)(*d)) = *((unsigned short *)(*s));/*16bits*/
-    *d += SCANSTEP, *s -= 12;
-    *((unsigned int *)(*d)) = *((unsigned int *)(*s));/*32bits*/
-    *d += 4, *s += 4;
-    *((unsigned int *)(*d)) = *((unsigned int *)(*s));/*32bits*/
-    *d += 4, *s += 4;
-    *((unsigned int *)(*d)) = *((unsigned int *)(*s));/*32bits*/
-    *d += 4, *s += 4;
-    *((unsigned short *)(*d)) = *((unsigned short *)(*s));/*16bits*/
-    *d += SCANSTEP, *s += 4;
-#else
-    *((unsigned int *)(*d)) = *((unsigned int *)(*s));/*32bits*/
-    *d += 4, *s += 4;
-    *((unsigned short *)(*d)) = *((unsigned short *)(*s));/*16bits*/
-    *d += 2, *s += 2;
-    *((unsigned char *)(*d)) = *((unsigned char *)(*s));/*8bits*/
-    *d += SCANSTEP, *s += 2;
-#endif
-}
-
-static void c_interface_print_char40( int x, int y, unsigned char c )
-{
-    int off;
-    unsigned char *d;
-    unsigned char *s;
-
-#ifdef _640x400
-    off = y * SCANWIDTH * 16 + x * 14 + 4;
-    s = interface_font + c * 128;
-#else
-    off = y * 320 * 8 + x * 7 + 1300;
-    s = interface_font + c * 64;
-#endif
-    d = GM + off;
-
-    c_interface_print_char40_line(&d,&s);
-    c_interface_print_char40_line(&d,&s);
-    c_interface_print_char40_line(&d,&s);
-    c_interface_print_char40_line(&d,&s);
-    c_interface_print_char40_line(&d,&s);
-    c_interface_print_char40_line(&d,&s);
-    c_interface_print_char40_line(&d,&s);
-    c_interface_print_char40_line(&d,&s);
-}
-
-/* -------------------------------------------------------------------------
-    c_interface_print_char()
-   ------------------------------------------------------------------------- */
-void c_interface_print_char( int x, int y, unsigned char c )
-{
-    c_interface_print_char40(x,y,c);
-}
-
-/* -------------------------------------------------------------------------
-    c_interface_print40()
-   ------------------------------------------------------------------------- */
-
-static void c_interface_print40( int x, int y, unsigned char *s )
-{
-    int		i;
-
-    for (i = x; *s; i++, s++)
-	c_interface_print_char40( i, y, *s );
+    video_loadfont_int(0x80,11,interface_glyphs);
 }
 
 /* -------------------------------------------------------------------------
     c_interface_print()
    ------------------------------------------------------------------------- */
-void c_interface_print( int x, int y, unsigned char *s )
+void c_interface_print( int x, int y, int cs, unsigned char *s )
 {
-    c_interface_print40(x,y,s);
+    int		i;
+
+    for (i = x; *s; i++, s++)
+	video_plotchar( i, y, cs, *s );
+ 
 }
 
 /* -------------------------------------------------------------------------
@@ -352,15 +108,13 @@ void c_interface_print( int x, int y, unsigned char *s )
    ------------------------------------------------------------------------- */
 
 void c_interface_redo_bottom() {
-    c_interface_textcolor( COLOR_LIGHT_RED, 0 );
-    c_interface_print( 1, 21,
+    
+    c_interface_print( 1, 21, 2,
 	" Use arrow keys (or Return) to modify "
 		       );
-    c_interface_print( 1, 22,
+    c_interface_print( 1, 22, 2,
 	" parameters. (Press ESC to exit menu) "
 		       );
-
-    c_interface_textcolor( COLOR_LIGHT_GREEN, 0 );
 }
 
 /* -------------------------------------------------------------------------
@@ -368,15 +122,12 @@ void c_interface_redo_bottom() {
    ------------------------------------------------------------------------- */
 
 void c_interface_redo_diskette_bottom() {
-    c_interface_textcolor( COLOR_LIGHT_RED, 0 );
-    c_interface_print( 1, 21,
+    c_interface_print( 1, 21, 2,
 	" Move: Arrows, PGUP, PGDN, HOME, END. "
 		       );
-    c_interface_print( 1, 22,
+    c_interface_print( 1, 22, 2,
 	" Return and 'w' select, ESC cancels.  "
 		       );
-
-    c_interface_textcolor( COLOR_LIGHT_GREEN, 0 );
 }
 
 /* -------------------------------------------------------------------------
@@ -530,16 +281,16 @@ int c_interface_is_nibblized(const char *name)
 
 int c_interface_disk_select(const struct dirent *e)
 {
-    static char		cmp[ 4096 ];
+    static char		cmp[ DISKSIZE ];
     size_t len;
     const char *p;
 
-    strcpy( cmp, disk_path );
-    strcat( cmp, "/" );
-    strcat( cmp, e -> d_name );
+    strncpy( cmp, disk_path, DISKSIZE );
+    strncat( cmp, "/", DISKSIZE );
+    strncat( cmp, e -> d_name, DISKSIZE );
 
     /* don't show disk in alternate drive */
-    if (!strcmp(cmp, file_name_6[altdrive]))
+    if (!strcmp(cmp, disk6.disk[altdrive].file_name))
 	return 0;
 
     /* show directories except '.' and '..' at toplevel. */
@@ -566,24 +317,12 @@ int c_interface_disk_select(const struct dirent *e)
 
 /* -------------------------------------------------------------------------
     c_interface_exit()
-    Redraw the 2 screens when exiting the menus.  We have to flag this unique
-    case so that we are forced to draw the screen (potentially ignoring what
-    the page indices are).
    ------------------------------------------------------------------------- */
 
 void c_interface_exit()
 {
-    /* update the current non-visual screen */
-    c_setscreen(!vmode_screen);
-    updating_screen = 1;/* flag unique case */
-    update_video_screen();
-    updating_screen = 0;
-
-    /* update the current visual page */
-    c_setscreen(vmode_screen);
-    updating_screen = 1;/* flag unique case */
-    update_video_screen();
-    updating_screen = 0;
+    video_setpage(!!(softswitches & SS_SCREEN));
+    video_redraw();
 }
 
 
@@ -627,16 +366,13 @@ void c_interface_select_diskette( int drive )
 
     screen[ 1 ][ 29 ] = (drive == 0) ? 'A' : 'B';
 
-    c_setpage( 0 );
-    c_setscreen( 0 );
+    video_setpage( 0 );
 
     c_interface_translate_screen( screen );
 
  NEXTDIR:
-    c_interface_textcolor( COLOR_LIGHT_RED, 0 );
     for (i = 0; i < 24; i++)
-	c_interface_print( 0, i, screen[ i ] );
-    c_interface_textcolor( COLOR_LIGHT_GREEN, 0 );
+	c_interface_print( 0, i, 2, screen[ i ] );
 
     altdrive = (drive == 0) ? 1 : 0;
     if (!strcmp("", disk_path))
@@ -650,7 +386,7 @@ void c_interface_select_diskette( int drive )
 	/* 1/17/97 NOTE: scandir (libc5.3.12) seems to freak on broken
            symlinks.  so you won't be able to read from some
            legitimate directories... */
-	c_interface_print( 6, 11, "Problem reading directory!" );
+	c_interface_print( 6, 11, 0, "Problem reading directory!" );
 	sprintf(disk_path, "/");
 	usleep(1500000);
 	c_interface_exit();
@@ -661,30 +397,32 @@ void c_interface_select_diskette( int drive )
 	curpos = entries - 1;
 
     for (;;) {
-        c_interface_textcolor( COLOR_LIGHT_GREEN, 0 );
         for (i = 0; i < 17; i++) {
 	    int		ent_no = curpos - 8 + i, slen;
 	    int		in_drive = 0;
 
 	    strcpy( temp, " " );
 	    if (ent_no >= 0 && ent_no < entries) {
-		sprintf(temp, "%s/%s", disk_path, namelist[ent_no]->d_name);
-		if (!strcmp(temp, file_name_6[drive]))
+		snprintf(temp, TEMPSIZE, "%s/%s",
+		         disk_path, namelist[ent_no]->d_name);
+		if (!strcmp(temp, disk6.disk[drive].file_name))
 		    in_drive = 1;
 
 		stat(temp, &statbuf);
 		if (S_ISDIR(statbuf.st_mode))
-		    sprintf(temp, " %s/", namelist[ ent_no ] -> d_name );
+		    snprintf(temp, TEMPSIZE, " %s/",
+		             namelist[ ent_no ] -> d_name );
 		else
-		    sprintf(temp, " %s", namelist[ ent_no ] -> d_name );
+		    snprintf(temp, TEMPSIZE, " %s",
+		             namelist[ ent_no ] -> d_name );
 	        if (c_interface_cut_name(temp)) {
-		    strcat(temp, " <gz>");
+		    strncat(temp, " <gz>", TEMPSIZE);
 		}
 		/* write protected disk in drive? */
-		else if ((in_drive) && (protected_6[drive]))
-		    strcat(temp, (drive == 0) ? " <r1>" : " <r2>");
+		else if ((in_drive) && (disk6.disk[drive].protected))
+		    strncat(temp, (drive == 0) ? " <r1>" : " <r2>", TEMPSIZE);
 		else if (in_drive)
-		    strcat(temp, (drive == 0) ? " <rw1>" : " <rw2>");
+		    strncat(temp, (drive == 0) ? " <rw1>" : " <rw2>", TEMPSIZE);
 	    }
 
 	    slen = strlen( temp );
@@ -692,13 +430,7 @@ void c_interface_select_diskette( int drive )
 	        temp[ slen++ ] = ' ';
 	    temp[ 38 ] = '\0';
 
-	    if (ent_no == curpos) {	/* color cursor position */
-		c_interface_textcolor(COLOR_LIGHT_GREEN, COLOR_MEDIUM_BLUE);
-		c_interface_print(1, i + 3, temp);
-		c_interface_textcolor(COLOR_LIGHT_GREEN, 0);
-	    }
-	    else
-		c_interface_print(1, i + 3, temp);
+	    c_interface_print(1, i + 3, ent_no == curpos, temp);
 	}
 
 	do
@@ -743,19 +475,23 @@ void c_interface_select_diskette( int drive )
 	{
 	    int len, cmpr = 0;
 
-            sprintf(temp, "%s/%s", disk_path, namelist[ curpos ] -> d_name );
+            snprintf(temp, TEMPSIZE, "%s/%s",
+                     disk_path, namelist[ curpos ] -> d_name );
 	    len = strlen(disk_path);
 
 	    /* handle disk currently in the drive */
-	    if (!strcmp(temp, file_name_6[drive])) {
+	    if (!strcmp(temp, disk6.disk[drive].file_name)) {
 		/* reopen disk, forcing write enabled */
 		if (toupper(ch) == 'W') {
 		    if (c_new_diskette_6(
-			drive, temp, compressed_6[drive], nibblized_6[drive], 0))
+			drive,
+                        temp, 
+                        disk6.disk[drive].compressed,
+			disk6.disk[drive].nibblized, 0))
 		    {
-			c_interface_print( 1, 21,
+			c_interface_print( 1, 21, 0,
 				   "  Disk is read and write protected.   " );
-			c_interface_print( 1, 22,
+			c_interface_print( 1, 22, 0,
 				   "                                      " );
 			usleep(1500000);
 			c_mygetch(1);
@@ -790,7 +526,8 @@ void c_interface_select_diskette( int drive )
 			disk_path[len] = '\0';
 		}
 		else if (strcmp(".", namelist[curpos]->d_name))
-		    sprintf(disk_path + len, "/%s", namelist[curpos]->d_name);
+		    snprintf(disk_path + len, DISKSIZE-len, "/%s",
+		             namelist[curpos]->d_name);
 		for (i = 0; i < entries; i++)
 		    free(namelist[ i ]);	/* clean up */
 		free(namelist);
@@ -801,14 +538,14 @@ void c_interface_select_diskette( int drive )
 	    if (c_interface_is_gz(temp))
 	    {
 		if ((pid = fork())) {	/* parent process */
-		    c_interface_print( 1, 21,
+		    c_interface_print( 1, 21, 0,
 			"            Uncompressing...          " );
-		    c_interface_print( 1, 22,
+		    c_interface_print( 1, 22, 0,
 			"                                      " );
 		    if (waitpid(pid, NULL, 0) == -1) {
-			c_interface_print( 1, 21,
-			"          Problem gunzip'ing          " );
-			c_interface_print( 1, 22,
+			c_interface_print( 1, 21, 0,
+ 			"          Problem gunzip'ing          " );
+			c_interface_print( 1, 22, 0,
 			"                                      " );
 			usleep(1500000);
 			c_mygetch(1);
@@ -820,19 +557,19 @@ void c_interface_select_diskette( int drive )
 		    if (execl("/bin/gzip", "/bin/gzip",
 			      "-d", temp, NULL) == -1)
 		    {
-			sprintf(temp, "%s", sys_errlist[errno]);
+			snprintf(temp, TEMPSIZE, "%s", sys_errlist[errno]);
 			perror("\tproblem");
-			c_interface_print( 1, 21,
+			c_interface_print( 1, 21, 0,
 			       "    Problem exec'ing /bin/gzip -d     " );
-			c_interface_print( 1, 22, temp);
+			c_interface_print( 1, 22, 0, temp);
 			usleep(1500000);
 			exit(-1);
 		    }
 		} else {
-		    sprintf(temp, "%s", sys_errlist[errno]);
-		    c_interface_print( 1, 21,
+		    snprintf(temp, TEMPSIZE, "%s", sys_errlist[errno]);
+		    c_interface_print( 1, 21, 0,
 			       "            Cannot fork!              " );
-		    c_interface_print( 1, 22, temp);
+		    c_interface_print( 1, 22, 0, temp);
 		    usleep(1500000);
 		    c_mygetch(1);
 		    c_interface_redo_diskette_bottom();
@@ -844,19 +581,19 @@ void c_interface_select_diskette( int drive )
 	    }
 
 	    /* gzip the last disk */
-	    if (compressed_6[ drive ]) {
+	    if (disk6.disk[drive].compressed) {
 		/* gzip the last disk if it was compressed_6 */
 
 		if ((pid = fork())) {	/* parent process */
 		    /* privileged mode - gzip in place */
-		    c_interface_print( 1, 21,
+		    c_interface_print( 1, 21, 0,
 			"      Compressing old diskette...     " );
-		    c_interface_print( 1, 22,
+		    c_interface_print( 1, 22, 0,
 			"                                      " );
 		    if (waitpid(pid, NULL, 0) == -1) {
-			c_interface_print( 1, 21,
+			c_interface_print( 1, 21, 0,
 			"           Problem gzip'ing           " );
-			c_interface_print( 1, 22,
+			c_interface_print( 1, 22, 0,
 			"                                      " );
 			usleep(1500000);
 			c_mygetch(1);
@@ -866,19 +603,19 @@ void c_interface_select_diskette( int drive )
 		} else if (!pid) {	/* child process */
 		    /* privileged mode - gzip in place */
 		    if (execl("/bin/gzip", "/bin/gzip",
-			      file_name_6[drive], NULL) == -1)
+			      disk6.disk[drive].file_name, NULL) == -1)
 		    {
-			c_interface_print( 1, 21,
+			c_interface_print( 1, 21, 0,
 			       "      Problem exec'ing /bin/gzip      " );
-			c_interface_print( 1, 22, temp);
+			c_interface_print( 1, 22, 0, temp);
 			usleep(1500000);
 			exit(-1);
 		    }
 		} else {
-		    sprintf(temp, "%s", sys_errlist[errno]);
-		    c_interface_print( 1, 21,
+		    snprintf(temp, TEMPSIZE, "%s", sys_errlist[errno]);
+		    c_interface_print( 1, 21, 0,
 			       "            Cannot fork!              " );
-		    c_interface_print( 1, 22, temp);
+		    c_interface_print( 1, 22, 0, temp);
 		    usleep(1500000);
 		    c_mygetch(1);
 		    c_interface_redo_diskette_bottom();
@@ -891,9 +628,9 @@ void c_interface_select_diskette( int drive )
 		drive, temp, cmpr, c_interface_is_nibblized(temp),
 		(toupper(ch) != 'W')))
 	    {
-		c_interface_print( 1, 21,
+		c_interface_print( 1, 21, 0,
 			"  Disk is read and write protected.   " );
-	        c_interface_print( 1, 22,
+	        c_interface_print( 1, 22, 0,
 			"                                      " );
 		usleep(1500000);
 		c_mygetch(1);
@@ -975,33 +712,25 @@ void c_interface_parameters()
 
     /* reset the x position, so we don't lose our cursor if path changes */
     cur_x = 0;
-    c_setpage( 0 );
-    c_setscreen( 0 );
+    video_setpage( 0 );
 
     c_interface_translate_screen( screen );
 
-    c_interface_textcolor( COLOR_LIGHT_RED, 0 );
     for (i = 0; i < 24; i++)
-	c_interface_print( 0, i, screen[ i ] );
-    c_interface_textcolor( COLOR_LIGHT_GREEN, 0 );
+	c_interface_print( 0, i, 2,screen[ i ] );
 
     for (;;) {
 	for (i = 0; i < 9; i++) {
 	    cur_off = option - 8;
 	    if (cur_off < 0) cur_off = 0;
 
-	    if (cur_y == i)
-		c_interface_textcolor( COLOR_LIGHT_GREEN,
-				       COLOR_MEDIUM_BLUE );
-	    c_interface_print( 1, 5 + i, options[ i + cur_off ] );
-	    if (cur_y == i)
-		c_interface_textcolor( COLOR_LIGHT_GREEN,
-				       0 );
-
+	    c_interface_print( 1, 5 + i, cur_y == i, options[ i + cur_off ] );
+				    
 	    switch (i + cur_off)
 	    {
 	    case 0:
-		sprintf(temp, "%03d", MAX_APPLE_DELAY + 1 - apple_speed);
+		snprintf(temp, TEMPSIZE, "%03d",
+		         MAX_APPLE_DELAY + 1 - cpu65_delay);
 		break;
 	    case 1:
 		strncpy(temp, disk_path + cur_pos, 24);
@@ -1068,7 +797,7 @@ void c_interface_parameters()
 
 	    pad_string(temp, ' ', 26);
 	    if (i+cur_off != 1)
-	        c_interface_print(14, 5 + i, temp);
+	        c_interface_print(14, 5 + i, 0, temp);
 	    else {
 	        int	j;
 
@@ -1077,33 +806,28 @@ void c_interface_parameters()
 		    {
 			if (temp[ j ] == '\0')
 			{
-		            c_interface_print_char( 14 + j, 5+i, ' ' );
+		            video_plotchar( 14 + j, 5+i, 0, ' ' );
 			    j++;
 			    break;
 			}
 			else
-		            c_interface_print_char( 14 + j, 5+i, temp[ j ] );
+		            video_plotchar( 14 + j, 5+i, 0, temp[ j ] );
 		    }
 		    else
 		    {
-			if (option == 1)
-		            c_interface_textcolor( COLOR_LIGHT_GREEN,
-				                   COLOR_MEDIUM_BLUE );
 			if (temp[ j ] == '\0')
 			{
-		            c_interface_print_char( 14 + j, 5+i, ' ' );
-			    if (option == 1)
-		                c_interface_textcolor( COLOR_LIGHT_GREEN, 0 );
+		            video_plotchar( 14 + j, 5+i, option==1,' ' );
 			    j++;
 			    break;
 			}
 			else
-		            c_interface_print_char( 14 + j, 5+i, temp[ j ] );
-			if (option == 1)
-		            c_interface_textcolor( COLOR_LIGHT_GREEN, 0 );
+		            video_plotchar( 14 + j, 5+i, option==1,
+                                                   temp[ j ]);
+			
 		    }
 		for (; j < 24; j++)
-		    c_interface_print_char( 14 + j, 5+i, ' ' );
+		    video_plotchar( 14 + j, 5+i, 0, ' ' );
 	     }
         }
 
@@ -1137,8 +861,8 @@ void c_interface_parameters()
 	else if (ch == kLEFT) {		/* Arrow left */
 	    switch (option) {
 		case 0:			/* inc speed */
-		    if (apple_speed < MAX_APPLE_DELAY)
-			apple_speed++;
+		    if (cpu65_delay < MAX_APPLE_DELAY)
+			cpu65_delay++;
 		    break;
 		case 1:			/* path */
 		    if (cur_x > 0)
@@ -1159,12 +883,6 @@ void c_interface_parameters()
 			color_mode = 4;
 		    else
 			--color_mode;
-		    if (color_mode == COLOR)
-			strict_color = 1;	/* strict colors */
-		    else if (color_mode == INTERP)
-			strict_color = 2;	/* strict interpolation */
-		    else
-			strict_color = 0;	/* lazy coloration */
 		    break;
 		case 4:			/* sound mode */
 		    if (sound_mode == 0)
@@ -1218,8 +936,8 @@ void c_interface_parameters()
 	else if (ch == kRIGHT) {		/* Arrow right */
 	    switch (option) {
 		case 0:			/* dec speed */
-		    if (apple_speed > 1)
-			apple_speed--;
+		    if (cpu65_delay > 1)
+			cpu65_delay--;
 		    break;
 		case 1:			/* path */
 		    if (cur_x < 23)
@@ -1242,12 +960,6 @@ void c_interface_parameters()
 		    color_mode++;
 		    if (color_mode > 4)
 			color_mode = 0;
-		    if (color_mode == COLOR)
-			strict_color = 1;	/* strict colors */
-		    else if (color_mode == INTERP)
-			strict_color = 2;	/* strict interpolation */
-		    else
-			strict_color = 0;	/* lazy coloration */
 		    break;
 		case 4:			/* sound mode */
 		    sound_mode++;
@@ -1298,7 +1010,7 @@ void c_interface_parameters()
 	}
 	else if (ch == kESC) {			/* exit menu */
 	    c_initialize_sound();
-	    c_initialize_hires_values();	/* redo colors */
+	    video_set(0);			/* redo colors */
 #ifdef PC_JOYSTICK
 	    if (joy_mode == JOY_PCJOY) {
 		c_close_joystick();		/* close the joystick */
@@ -1311,7 +1023,8 @@ void c_interface_parameters()
 #endif
 	    /* reboot machine if different */
 	    if (current_mode != apple_mode)
-		exception_flag = RebootSig;	/* flag reset state */
+		cpu65_interrupt(RebootSig);
+
 	    c_interface_exit();
 	    return;
 	}
@@ -1320,11 +1033,11 @@ void c_interface_parameters()
 	    if (ch >= ' ' && ch < 127 && option == PATH_OPTION) {
 		int		i;
 
-		strcpy(temp, disk_path);		
+		strncpy(temp, disk_path, TEMPSIZE);		
 		for (i = strlen(temp); i >= cur_pos + cur_x; i--)
 		    temp[ i + 1 ] = temp[ i ];
 		temp[ cur_pos + cur_x ] = ch;
-		strcpy(disk_path, temp);
+		strncpy(disk_path, temp, DISKSIZE);
 		if (cur_x < 23)
 		    cur_x++;
 		else
@@ -1355,7 +1068,7 @@ void c_interface_parameters()
 #endif
 	    /* save settings */
 	    if ((ch == 13) && (option == SAVE_SETTINGS)) {
-		c_save_settings();
+		save_settings();
 	    }
 
 	    /* quit apple II simulator */
@@ -1363,7 +1076,7 @@ void c_interface_parameters()
 		int ch;
 
 		c_interface_print(
-		    1, 22, "          Are you sure? (Y/N)         " );
+		    1, 22, 0, "          Are you sure? (Y/N)         " );
 		while ((ch = c_mygetch(1)) == -1) ;
 		ch = toupper(ch);
 		if (ch == 'Y')
@@ -1375,12 +1088,12 @@ void c_interface_parameters()
 #endif
 		    printf("Linux! ...and there were much rejoicing! "
 			   "oyeeeeh...\n");
-		    c_shutdown_video();
-		    exit( 0 );/* shouldn't get here */
+		    video_shutdown();
+		    exit( 0 );
 		}
-                c_interface_textcolor( COLOR_LIGHT_RED, 0 );
-	        c_interface_print( 0, 22, screen[ 22 ] );
-                c_interface_textcolor( COLOR_LIGHT_GREEN, 0 );
+                
+	        c_interface_print( 0, 22, 2, screen[ 22 ] );
+                
 	    }
 	}
     }
@@ -1422,14 +1135,12 @@ void c_interface_words()
 
     int		i;
 
-    c_setpage( 0 );
-    c_setscreen( 0 );
+    video_setpage( 0 );
 
     c_interface_translate_screen( screen );
 
-    c_interface_textcolor( COLOR_LIGHT_RED, 0 );
     for (i = 0; i < 24; i++)
-	c_interface_print( 0, i, screen[ i ] );
+	c_interface_print( 0, i, 2, screen[ i ] );
 
     while (c_mygetch(1) == -1) { }
     c_interface_exit();
@@ -1446,24 +1157,24 @@ void c_interface_keyboard_layout()
       { "||||||||||||||||||||||||||||||||||||||||",
 	"|     Apple II+ US Keyboard Layout     |",
 	"||||||||||||||||||||||||||||||||||||||||",
-	"| Keyboard:                            |",
 	"|                                      |",
 	"| 1! 2\" 3# 4$ 5% 6& 7' 8( 9) 0 :* -=   |",
-	"|  Q  W  E  R  T  Y  U  I  O  P@ RP CR |",
+	"|  Q  W  E  R  T  Y  U  I  O  P@    CR |",
 	"|   A  S  D  F  G  H  J  K  L  ;+ <- ->|",
 	"|    Z  X  C  V  B  N^ M ,< .> /?      |",
 	"| Where <- -> are the left and right   |",
-	"| arrow keys respectively. RP is the   |",
-	"| \"repeat\" key.                        |",
-	"||||||||||||||||||||||||||||||||||||||||",
-	"| Joystick emulation on numeric keypad |",
+	"| arrow keys respectively.             |",
 	"|                                      |",
+	"| Joystick emulation on numeric keypad |",
 	"| 7  8  9  for various directions.     |",
 	"| 4     6  Press 5 to center linear    |",
 	"| 1  2  3  joystick.                   |",
 	"|                                      |",
-	"| Delete, End and Page Down emulates   |",
-	"| pushbuttons 0, 1 and 2 respectively. |",
+	"| Ctrl-PrintScrn/SysRq is REBOOT.      |",
+	"| Ctrl-Pause/Break is RESET.           |",
+	"| Pause/Break alone pauses emulation.  |",
+	"| Alt Left and Alt Right are Apple     |",
+	"| Keys (Joystick buttons 0 & 1).       |",
 	"||||||||||||||||||||||||||||||||||||||||",
 	"|       (Press any key to exit)        |",
 	"||||||||||||||||||||||||||||||||||||||||" };
@@ -1476,21 +1187,21 @@ void c_interface_keyboard_layout()
 	"|                                      |",
 	"|1! 2@ 3# 4$ 5% 6^ 7& 8* 9( 0) -_ =+ dl|",
 	"| Q  W  E  R  T  Y  U  I  O P [{ ]} \\| |",
-	"|cp A  S  D  F  G  H  J  K  L ;: '\" rt |",
+	"|cp A  S  D  F  G  H  J  K  L ;: '\" CR |",
 	"|sh  Z  X  C  V  B  N  M ,< .> /?   sh |",
 	"|ctrl                                  |",
 	"|                                      |",
-	"| Where dl is DEL, cp is CAPS, rt is   |",
-	"| RET, sh is SHIFT, ctrl is CONTROL.   |",
+	"| Where dl is DEL, cp is CAPS, CR is   |",
+	"| RETURN, sh is SHIFT, ctrl is CONTROL.|",
 	"| Arrow keys are as is.                |",
+	"| Joystick emulation is same as the    |",
+	"| ][+.                                 |",
 	"|                                      |",
-	"| PrintScrn/SysRq is RESET.            |",
-	"| Pause/Break is REBOOT.               |",
-	"| Delete, End and Page Down emulates   |",
-	"| pushbuttons 0, 1 and 2 and the Apple |",
-	"| keys respectively. Joystick emulation|",
-	"| is the same as the ][+.              |",
-	"|                                      |",
+	"| Ctrl-PrintScrn/SysRq is REBOOT.      |",
+	"| Ctrl-Pause/Break is RESET.           |",
+	"| Pause/Break alone pauses emulation.  |",
+	"| Alt Left and Alt Right are Apple     |",
+	"| Keys (Joystick buttons 0 & 1).       |",
 	"||||||||||||||||||||||||||||||||||||||||",
 	"|       (Press any key to exit)        |",
 	"||||||||||||||||||||||||||||||||||||||||" };
@@ -1498,22 +1209,19 @@ void c_interface_keyboard_layout()
 
     int		i;
 
-    c_setpage( 0 );
-    c_setscreen( 0 );
+    video_setpage( 0 );
 
 #ifdef APPLE_IIE
     if (apple_mode == 2) {
-	c_interface_translate_screen(screen2);
-	c_interface_textcolor( COLOR_LIGHT_RED, 0 );
+	c_interface_translate_screen(screen2); 
 	for (i = 0; i < 24; i++)
-	    c_interface_print( 0, i, screen2[ i ] );
+	    c_interface_print( 0, i, 2, screen2[ i ] );
     } else
 #endif
     {
 	c_interface_translate_screen(screen1);
-	c_interface_textcolor( COLOR_LIGHT_RED, 0 );
 	for (i = 0; i < 24; i++)
-	    c_interface_print( 0, i, screen1[ i ] );
+	    c_interface_print( 0, i, 2, screen1[ i ] );
     }
 
     while (c_mygetch(1) == -1) { }
