@@ -4,7 +4,7 @@
  * Copyright 1994 Alexander Jean-Claude Bottema
  * Copyright 1995 Stephen Lee
  * Copyright 1997, 1998 Aaron Culliney
- * Copyright 1998, 1999 Michael Deutschmann
+ * Copyright 1998, 1999, 2000 Michael Deutschmann
  *
  * This software package is subject to the GNU General Public License
  * version 2 or later (your choice) as published by the Free Software 
@@ -21,10 +21,11 @@
 #include <linux/keyboard.h>
 
 #include "keys.h"
-#include "apple2.h"
 #include "misc.h"
 #include "video.h"
 #include "interface.h"
+#include "cpu.h"
+#include "prefs.h"
 
 /* from misc.c */
 extern uid_t user, privileged;
@@ -42,19 +43,14 @@ unsigned char	joy_button0 = 0;
 unsigned char	joy_button1 = 0;
 unsigned char	joy_button2 = 0;
 
-/* from joystick.c */
 #ifdef PC_JOYSTICK
-#include "linux/joystick.h"
-extern int js_fd;
-extern struct JS_DATA_TYPE js;
-extern int js_center_x, js_center_y, js_offset_x, js_offset_y;
-extern float js_adjustlow_x, js_adjustlow_y, js_adjusthigh_x, js_adjusthigh_y;
+#include <linux/joystick.h>
 int x_val, y_val;
 #endif
 
-static int		menu_key = -1;
-static int		flash_cnt = 0;
+static int		next_key = -1;
 static char		caps_lock = 1;	/* is enabled */
+static int		in_mygetch = 0;
 
 /* ----------------------------------------------------
     Keymap. Mapping scancodes to Apple II+ US Keyboard
@@ -67,14 +63,14 @@ static int apple_ii_keymap_plain[128] =
       'D', 'F', 'G', 'H', 'J', 'K', 'L', ';',   /* 32-39   */
       8  , -1 , -1 , 21 , 'Z', 'X', 'C', 'V',   /* 40-47   */
       'B', 'N', 'M', ',', '.', '/', -1 , -1 ,   /* 48-55   */
-      -1 , ' ', -1 , kF1 , kF2 , kF3 , kF4 , kF5 ,   /* 56-63   */
+      JB0 , ' ', -1 , kF1 , kF2 , kF3 , kF4 , kF5 ,   /* 56-63   */
       kF6 , kF7 , kF8 , kF9 , kF10, -1 , -1 , JUL,   /* 64-71   */
       J_U, JUR, S_D, J_L, J_C, J_R, S_I, JDL,   /* 72-79   */
       J_D, JDR, -1 , -1 , -1 , kF11, kF12, -1 ,   /* 80-87   */
       -1 , -1 , -1 , -1 , -1 , -1 , -1 , -1 ,   /* 88-95   */
-      -1 , -1 , -1 , RST, -1 , -1 , -1 , -1 ,   /* 96-103  */
-      -1 , 8  , 21 , JB1, -1 , JB2, -1 , JB0,   /* 104-111 */
-      -1 , -1 , -1 , -1 , -1 , -1 , -1 , BOT,   /* 112-119 */
+      -1 , -1 , -1 , -1, JB1 , RST , kHOME , -1 ,   /* 96-103  */
+      kPGUP , 8  , 21 , kEND , -1 , kPGDN, JB2 , -1,   /* 104-111 */
+      -1 , -1 , -1 , -1 , -1 , -1 , -1 , kF4 /* (pause) */,   /* 112-119 */
       -1 , -1 , -1 , -1 , -1 , -1 , -1 , -1 };  /* 120-127 */
 
 static int apple_ii_keymap_ctrl[128] =
@@ -85,14 +81,14 @@ static int apple_ii_keymap_ctrl[128] =
       4  , 6  , 7  , 8  , 10 , 11 , 12 , ';',   /* 32-39   */
       8  , -1 , -1 , 21 , 26 , 24 , 3  , 22 ,   /* 40-47   */
       2  , 14 , 13 , ',', '.', '/', -1 , -1 ,   /* 48-55   */
-      -1 , ' ', -1 , kF1 , kF2 , kF3 , kF4 , kF5 ,   /* 56-63   */
+      JB0 , ' ', -1 , kF1 , kF2 , kF3 , kF4 , kF5 ,   /* 56-63   */
       kF6 , kF7 , kF8 , kF9 , kF10, -1 , -1 , JUL,   /* 64-71   */
       J_U, JUR, S_D, J_L, J_C, J_R, S_I, JDL,   /* 72-79   */
       J_D, JDR, -1 , -1 , -1 , kF11, kF12, -1 ,   /* 80-87   */
       -1 , -1 , -1 , -1 , -1 , -1 , -1 , -1 ,   /* 88-95   */
-      -1 , -1 , -1 , RST, -1 , -1 , -1 , -1 ,   /* 96-103  */
-      -1 , 8  , 21 , JB1, -1 , JB2, -1 , JB0,   /* 104-111 */
-      -1 , -1 , -1 , -1 , -1 , -1 , -1 , BOT,   /* 112-119 */
+      -1 , -1 , -1 , BOT, JB1 , RST , kHOME , -1 ,   /* 96-103  */
+      kPGUP , 8  , 21 , kEND, -1 , kPGDN, JB2 , -1,   /* 104-111 */
+      -1 , -1 , -1 , -1 , -1 , -1 , -1 , kF4 /* pause */,   /* 112-119 */
       -1 , -1 , -1 , -1 , -1 , -1 , -1 , -1 };  /* 120-127 */
 
 static int apple_ii_keymap_shifted[128] =
@@ -103,14 +99,14 @@ static int apple_ii_keymap_shifted[128] =
       'D', 'F', 'G', 'H', 'J', 'K', 'L', '+',   /* 32-39   */
       8  , -1 , -1 , 21 , 'Z', 'X', 'C', 'V',   /* 40-47   */
       'B', '^', 'M', '<', '>', '?', -1 , -1 ,   /* 48-55   */
-      -1 , ' ', -1 , kF1 , kF2 , kF3 , kF4 , kF5 ,   /* 56-63   */
+      JB0 , ' ', -1 , kF1 , kF2 , kF3 , kF4 , kF5 ,   /* 56-63   */
       kF6 , kF7 , kF8 , kF9 , kF10, -1 , -1 , JUL,   /* 64-71   */
       J_U, JUR, S_D, J_L, J_C, J_R, S_I, JDL,   /* 72-79   */
       J_D, JDR, -1 , -1 , -1 , kF11, kF12, -1 ,   /* 80-87   */
       -1 , -1 , -1 , -1 , -1 , -1 , -1 , -1 ,   /* 88-95   */
-      -1 , -1 , -1 , RST, -1 , -1 , -1 , -1 ,   /* 96-103  */
-      -1 , 8  , 21 , JB1, -1 , JB2, -1 , JB0,   /* 104-111 */
-      -1 , -1 , -1 , -1 , -1 , -1 , -1 , BOT,   /* 112-119 */
+      -1 , -1 , -1 , -1 , JB1 , RST , kHOME , -1 ,   /* 96-103  */
+      kPGUP , 8  , 21 , kEND, -1 , kPGDN, JB2 , -1,   /* 104-111 */
+      -1 , -1 , -1 , -1 , -1 , -1 , -1 , kF4 /* pause */,   /* 112-119 */
       -1 , -1 , -1 , -1 , -1 , -1 , -1 , -1 };  /* 120-127 */
 
 #ifdef APPLE_IIE
@@ -125,14 +121,14 @@ static int apple_iie_keymap_plain[128] =
       'd', 'f', 'g', 'h', 'j', 'k', 'l', ';',   /* 32-39   */
      '\'', '`', -1 ,'\\', 'z', 'x', 'c', 'v',   /* 40-47   */
       'b', 'n', 'm', ',', '.', '/', -1 , -1 ,   /* 48-55   */
-      -1 , ' ', -1 , kF1 , kF2 , kF3 , kF4 , kF5 ,   /* 56-63   */
+      JB0 , ' ', -1 , kF1 , kF2 , kF3 , kF4 , kF5 ,   /* 56-63   */
       kF6 , kF7 , kF8 , kF9 , kF10, -1 , -1 , JUL,   /* 64-71   */
       J_U, JUR, S_D, J_L, J_C, J_R, S_I, JDL,   /* 72-79   */
       J_D, JDR, -1 , -1 , -1 , kF11, kF12, -1 ,   /* 80-87   */
       -1 , -1 , -1 , -1 , -1 , -1 , -1 , -1 ,   /* 88-95   */
-      -1 , -1 , -1 , RST, -1 , -1 , -1 , 11 ,   /* 96-103  */
-      -1 , 8  , 21 , JB1, 10 , JB2, 127, JB0,   /* 104-111 */
-      -1 , -1 , -1 , -1 , -1 , -1 , -1 , BOT,   /* 112-119 */
+      -1 , -1 , -1 , -1, JB1 , RST , kHOME , 11 ,   /* 96-103  */
+      kPGUP , 8  , 21 , kEND, 10 , kPGDN, JB2, 127,   /* 104-111 */
+      -1 , -1 , -1 , -1 , -1 , -1 , -1 , kF4 /* pause */,   /* 112-119 */
       -1 , -1 , -1 , -1 , -1 , -1 , -1 , -1 };  /* 120-127 */
 
 static int apple_iie_keymap_ctrl[128] =
@@ -143,14 +139,14 @@ static int apple_iie_keymap_ctrl[128] =
       4  , 6  , 7  , 8  , 10 , 11 , 12 , ';',   /* 32-39   */
      '\'', '`', -1 ,'\\', 26 , 24 , 3  , 22 ,   /* 40-47   */
       2  , 14 , 13 , ',', '.', '/', -1 , -1 ,   /* 48-55   */
-      -1 , ' ', -1 , kF1 , kF2 , kF3 , kF4 , kF5 ,   /* 56-63   */
+      JB0 , ' ', -1 , kF1 , kF2 , kF3 , kF4 , kF5 ,   /* 56-63   */
       kF6 , kF7 , kF8 , kF9 , kF10, -1 , -1 , JUL,   /* 64-71   */
       J_U, JUR, S_D, J_L, J_C, J_R, S_I, JDL,   /* 72-79   */
       J_D, JDR, -1 , -1 , -1 , kF11, kF12, -1 ,   /* 80-87   */
       -1 , -1 , -1 , -1 , -1 , -1 , -1 , -1 ,   /* 88-95   */
-      -1 , -1 , -1 , RST, -1 , -1 , -1 , 11 ,   /* 96-103  */
-      -1 , 8  , 21 , JB1, 10 , JB2, 127, JB0,   /* 104-111 */
-      -1 , -1 , -1 , -1 , -1 , -1 , -1 , BOT,   /* 112-119 */
+      -1 , -1 , -1 , BOT, JB1 , RST , kHOME , 11 ,   /* 96-103  */
+      kPGUP , 8  , 21 , kEND, 10 , kPGDN, JB2, 127,   /* 104-111 */
+      -1 , -1 , -1 , -1 , -1 , -1 , -1 , kF4 /* pause */,   /* 112-119 */
       -1 , -1 , -1 , -1 , -1 , -1 , -1 , -1 };  /* 120-127 */
 
 static int apple_iie_keymap_shifted[128] =
@@ -161,14 +157,14 @@ static int apple_iie_keymap_shifted[128] =
       'D', 'F', 'G', 'H', 'J', 'K', 'L', ':',   /* 32-39   */
       '"', '~', -1 , '|', 'Z', 'X', 'C', 'V',   /* 40-47   */
       'B', 'N', 'M', '<', '>', '?', -1 , -1 ,   /* 48-55   */
-      -1 , ' ', -1 , kF1 , kF2 , kF3 , kF4 , kF5 ,   /* 56-63   */
+      JB0 , ' ', -1 , kF1 , kF2 , kF3 , kF4 , kF5 ,   /* 56-63   */
       kF6 , kF7 , kF8 , kF9 , kF10, -1 , -1 , JUL,   /* 64-71   */
       J_U, JUR, S_D, J_L, J_C, J_R, S_I, JDL,   /* 72-79   */
       J_D, JDR, -1 , -1 , -1 , kF11, kF12, -1 ,   /* 80-87   */
       -1 , -1 , -1 , -1 , -1 , -1 , -1 , -1 ,   /* 88-95   */
-      -1 , -1 , -1 , RST, -1 , -1 , -1 , 11 ,   /* 96-103  */
-      -1 , 8  , 21 , JB1, 10 , JB2, 127, JB0,   /* 104-111 */
-      -1 , -1 , -1 , -1 , -1 , -1 , -1 , BOT,   /* 112-119 */
+      -1 , -1 , -1 , -1 , JB1 , RST , kHOME , 11 ,   /* 96-103  */
+      kPGUP , 8  , 21 , kEND, 10 , kPGDN, JB2, 127,   /* 104-111 */
+      -1 , -1 , -1 , -1 , -1 , -1 , -1 , kF4 /* pause */,   /* 112-119 */
       -1 , -1 , -1 , -1 , -1 , -1 , -1 , -1 };  /* 120-127 */
 
 static int apple_iie_keymap_caps[128] =
@@ -179,14 +175,14 @@ static int apple_iie_keymap_caps[128] =
       'D', 'F', 'G', 'H', 'J', 'K', 'L', ';',   /* 32-39   */
      '\'', '`', -1 ,'\\', 'Z', 'X', 'C', 'V',   /* 40-47   */
       'B', 'N', 'M', ',', '.', '/', -1 , -1 ,   /* 48-55   */
-      -1 , ' ', -1 , kF1 , kF2 , kF3 , kF4 , kF5 ,   /* 56-63   */
+      JB0 , ' ', -1 , kF1 , kF2 , kF3 , kF4 , kF5 ,   /* 56-63   */
       kF6 , kF7 , kF8 , kF9 , kF10, -1 , -1 , JUL,   /* 64-71   */
       J_U, JUR, S_D, J_L, J_C, J_R, S_I, JDL,   /* 72-79   */
       J_D, JDR, -1 , -1 , -1 , kF11, kF12, -1 ,   /* 80-87   */
       -1 , -1 , -1 , -1 , -1 , -1 , -1 , -1 ,   /* 88-95   */
-      -1 , -1 , -1 , RST, -1 , -1 , -1 , 11 ,   /* 96-103  */
-      -1 , 8  , 21 , JB1, 10 , JB2, 127, JB0,   /* 104-111 */
-      -1 , -1 , -1 , -1 , -1 , -1 , -1 , BOT,   /* 112-119 */
+      -1 , -1 , -1 , -1, JB1 , RST , kHOME , 11 ,   /* 96-103  */
+      kPGUP , 8  , 21 , kEND, 10 , kPGDN, JB2, 127,   /* 104-111 */
+      -1 , -1 , -1 , -1 , -1 , -1 , -1 , kF4 /* pause */,   /* 112-119 */
       -1 , -1 , -1 , -1 , -1 , -1 , -1 , -1 };  /* 120-127 */
 
 static int apple_iie_keymap_shift_ctrl[128] =
@@ -197,14 +193,14 @@ static int apple_iie_keymap_shift_ctrl[128] =
       4  , 6  , 7  , 8  , 10 , 11 , 12 , ';',   /* 32-39   */
      '\'', '`', 28 , -1 , 26 , 24 , 3  , 22 ,   /* 40-47   */
       2  , 14 , 13 , ',', '.', '/', -1 , -1 ,   /* 48-55   */
-      -1 , ' ', -1 , kF1 , kF2 , kF3 , kF4 , kF5 ,   /* 56-63   */
+      JB0 , ' ', -1 , kF1 , kF2 , kF3 , kF4 , kF5 ,   /* 56-63   */
       kF6 , kF7 , kF8 , kF9 , kF10, -1 , -1 , JUL,   /* 64-71   */
       J_U, JUR, S_D, J_L, J_C, J_R, S_I, JDL,   /* 72-79   */
       J_D, JDR, -1 , -1 , -1 , kF11, kF12, -1 ,   /* 80-87   */
       -1 , -1 , -1 , -1 , -1 , -1 , -1 , -1 ,   /* 88-95   */
-      -1 , -1 , -1 , RST, -1 , -1 , -1 , 11 ,   /* 96-103  */
-      -1 , 8  , 21 , JB1, 10 , JB2, 127, JB0,   /* 104-111 */
-      -1 , -1 , -1 , -1 , -1 , -1 , -1 , BOT,   /* 112-119 */
+      -1 , -1 , -1 , BOT, JB1 , RST , kHOME , 11 ,   /* 96-103  */
+      kPGUP , 8  , 21 , kEND, 10 , kPGDN, JB2, 127,   /* 104-111 */
+      -1 , -1 , -1 , -1 , -1 , -1 , -1 , kF4 /* pause */,   /* 112-119 */
       -1 , -1 , -1 , -1 , -1 , -1 , -1 , -1 };  /* 120-127 */
 #endif
 
@@ -222,87 +218,66 @@ static char			key_pressed[ 256 ];
  * 	-polls PC Joystick
  * 	-update palette for flashing text
  * ------------------------------------------------------------------------- */
-void c_periodic_update() {
+void c_periodic_update(int dummysig) {
+    int current_key;
 
-#ifdef DEBUGGER
-    /* debugger hooks - proper exception_flag is set if we've already
-       got a kF7 */
-    if ((menu_key >= 0) || (exception_flag & EnterDebugSig))
-    {
-	int key = (exception_flag & EnterDebugSig) ? kF7 : menu_key;
-#else
-    if (menu_key >= 0)
-    {
-	int key = menu_key;
-#endif
-	menu_key = -1;/* got it */
-	switch (key) {
+    video_sync(0);
+
+    if (next_key >= 0)
+    {	
+	current_key = next_key;
+	next_key = -1;
+
+	if (current_key < 128) {
+	    apple_ii_64k[0][0xC000] = current_key | 0x80;
+	    apple_ii_64k[1][0xC000] = current_key | 0x80;
+	} else switch (current_key) {
+	case RST:
+	    cpu65_interrupt(ResetSig);
+	    break;
+	case BOT:
+	    cpu65_interrupt(RebootSig);
+	    break;
+	case J_C:
+	    joy_x = joy_center_x;
+	    joy_y = joy_center_y;
+	    break;
 	case kF1:
-	    c_keyboard_off();
 	    c_interface_select_diskette( 0 );
-	    c_keyboard_on();
 	    break;
 	case kF2:
-	    c_keyboard_off();
 	    c_interface_select_diskette( 1 );
-	    c_keyboard_on();
 	    break;
 	case kF4:
-	    c_keyboard_off();
 	    while (c_mygetch(1) == -1) { }/*busy loop*/
-	    c_keyboard_on();
 	    break;
 	case kF5:
-	    c_keyboard_off();
 	    c_interface_keyboard_layout();
-	    c_keyboard_on();
 	    break;
 #ifdef DEBUGGER
 	case kF7:
-	    /* set exception_flag and return to complete execution of
-               current 6502 instruction */
-	    if (!(exception_flag & EnterDebugSig)) {
-		exception_flag = EnterDebugSig;
-		break;
-	    }
-	    exception_flag &= ~EnterDebugSig;
-	    c_keyboard_off();
-	    c_do_debugging();
-	    c_keyboard_on();
-	  break;
+	    cpu65_interrupt(EnterDebugSig);
+	    break;
 #endif
 #if 0
 	case kF8:
-	    c_keyboard_off();
 	    c_interface_words();
-	    c_keyboard_on();
 	    break;
 #endif
 	case kF9:
 	    if (max_speed != 0)
-	        apple_speed = max_speed, max_speed = 0;
+	        cpu65_delay = max_speed, max_speed = 0;
 	    else
-	        max_speed = apple_speed, apple_speed = 1;
+	        max_speed = cpu65_delay, cpu65_delay = 1;
 	    break;
 	case kF10:
 	    if (max_speed != 0)
-		apple_speed = max_speed, max_speed = 0;
-	    c_keyboard_off();
+		cpu65_delay = max_speed, max_speed = 0;
 	    c_interface_parameters();
-	    c_keyboard_on();
 	    break;
 	}
     }
 
-    /* cursor */
-    flash_cnt += (int)TIMER_DELAY;
-    if (flash_cnt > 333333) {
-	flash_cnt = 0;
-	c_flash_cursor(flash_cnt);
-    } else if (flash_cnt == (200000 / TIMER_DELAY) * TIMER_DELAY) {
-	c_flash_cursor(flash_cnt);
-    }
-
     /* simulated joystick */
     if (joy_mode == JOY_KYBD) {
 	if (key_pressed[ SCODE_J_U ])
@@ -375,6 +350,18 @@ void c_periodic_update() {
         joy_x = joy_y = 256;
 }
 
+#ifdef DEBUGGER
+/* Called from cpu code.  Perhaps should be moved to misc.c, but was 
+ * abstracted from function above...
+ */
+void enter_debugger(void)
+{
+     setitimer(ITIMER_VIRTUAL,&timer_off,0);
+     c_do_debugging();
+     setitimer(ITIMER_VIRTUAL,&timer_on,0);
+}
+#endif /* DEBUGGER */
+
 /* -------------------------------------------------------------------------
     void c_read_raw_key() : handle a scancode
    ------------------------------------------------------------------------- */
@@ -383,7 +370,7 @@ void c_read_raw_key(int scancode, int pressed) {
 
     /* determine which key mapping to use */
 #ifdef APPLE_IIE
-    if (apple_mode == IIE_MODE) {
+    if (apple_mode == IIE_MODE || in_mygetch) {
 	/* set/reset caps lock */
 	if (key_pressed[ SCODE_CAPS ])
 	    caps_lock = !caps_lock;
@@ -420,23 +407,8 @@ void c_read_raw_key(int scancode, int pressed) {
     if (pressed) {
 
 	key_pressed[ scancode ] = 1;
-	if ((unsigned int)(keymap[ scancode ]) < 128) {
-	    apple_ii_64k[0][0xC000] = keymap[scancode] | 0x80;
-	    apple_ii_64k[1][0xC000] = keymap[scancode] | 0x80;
-	    return;
-	}
 
 	switch (keymap[ scancode ]) {
-	case RST:
-	    exception_flag = ResetSig;
-	    break;
-	case BOT:
-	    exception_flag = RebootSig;
-	    break;
-	case J_C:
-	    joy_x = joy_center_x;
-	    joy_y = joy_center_y;
-	    break;
 	case JB0:
 	    joy_button0 = 0xff;/* open apple */
 	    break;
@@ -447,15 +419,9 @@ void c_read_raw_key(int scancode, int pressed) {
 	    joy_button2 = 0xff;/* unused? */
 	    break;
 	default:
+	    next_key = keymap[scancode];
 	    break;
 	}
-
-	/* menu keys */
-	if ((keymap[ scancode ] >= kF1) &&
-	    (keymap[ scancode ] <= kF10))
-	{
-	    menu_key = keymap[ scancode ];
-	}
     }
 
     /* key is released */
@@ -474,3 +440,22 @@ void c_read_raw_key(int scancode, int pressed) {
 	}
     }
 }
+
+int c_mygetch(int block)
+{
+    int retval;
+
+    in_mygetch = 1;
+
+    if (block) while (next_key == -1)
+        video_sync(1);
+    else
+        video_sync(0);
+
+    in_mygetch = 0;
+
+    retval = next_key;
+    next_key = -1;
+
+    return retval;
+}
