@@ -4,7 +4,7 @@
  * Copyright 1994 Alexander Jean-Claude Bottema
  * Copyright 1995 Stephen Lee
  * Copyright 1997, 1998 Aaron Culliney
- * Copyright 1998, 1999 Michael Deutschmann
+ * Copyright 1998, 1999, 2000 Michael Deutschmann
  *
  * This software package is subject to the GNU General Public License
  * version 2 or later (your choice) as published by the Free Software 
@@ -17,7 +17,6 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <unistd.h>
-#include <signal.h>
 #include <string.h>
 #include <sys/types.h>
 #include <sys/ipc.h>
@@ -31,7 +30,6 @@
 
 
 #include "video.h"
-#include "apple2.h"
 #include "misc.h"
 #include "keys.h"
 
@@ -39,8 +37,6 @@
 static unsigned char	*svga_GM;
 static unsigned char	vga_mem_page_0[SCANWIDTH*SCANHEIGHT];		/* page0 framebuffer */
 static unsigned char	vga_mem_page_1[SCANWIDTH*SCANHEIGHT];		/* page1 framebuffer */
-unsigned char	odd_colors[2] = {COLOR_LIGHT_PURPLE, COLOR_LIGHT_BLUE};
-unsigned char	even_colors[2] = {COLOR_LIGHT_GREEN, COLOR_LIGHT_RED};
 
 static Display *display;
 static Window win;
@@ -60,41 +56,31 @@ XShmSegmentInfo	xshminfo;
 int		xshmeventtype;
 
 
-
 /* -------------------------------------------------------------------------
-    c_setpage(p):    Set SVGA 64k page and update GM
-                     (Graph Memory base address)
+    video_setpage(p):    Switch to screen page p
    ------------------------------------------------------------------------- */
-void c_setpage(int p)
+void video_setpage(int p)
 {
-    svga_GM = image->data;
-    GM = (crnt_visual_svga_page == p) ? svga_GM :
-	((p == 0) ? vga_mem_page_0 : vga_mem_page_1);
-    vmode_active = p;
-}
+    if (p == video__current_page) return;
 
-/* -------------------------------------------------------------------------
-    c_setscreen(p):    Switch to screen page p
-   ------------------------------------------------------------------------- */
-void c_setscreen(int m)
-{
-    if (m != crnt_visual_svga_page) {
-	memcpy( ((crnt_visual_svga_page == 0) ?
-		vga_mem_page_0 : vga_mem_page_1),
-		svga_GM,
-		SCANWIDTH*SCANHEIGHT
-		);
-	memcpy( svga_GM, ((m == 0) ? vga_mem_page_0 : vga_mem_page_1),
-		SCANWIDTH*SCANHEIGHT
-		);
-	GM = (vmode_active == m) ? svga_GM :
-	    ((vmode_active == 0) ?
-	     vga_mem_page_0 : vga_mem_page_1);
-	crnt_visual_svga_page = m;
+    if (p)
+    {
+	memcpy(vga_mem_page_0,svga_GM,SCANWIDTH*SCANHEIGHT);
+	memcpy(svga_GM,vga_mem_page_1,SCANWIDTH*SCANHEIGHT);
+	video__current_page = 1;
+	video__fb1 = vga_mem_page_0;
+	video__fb2 = svga_GM;
+    }
+    else
+    {
+	memcpy(vga_mem_page_1,svga_GM,SCANWIDTH*SCANHEIGHT);
+	memcpy(svga_GM,vga_mem_page_0,SCANWIDTH*SCANHEIGHT);
+	video__current_page = 0;
+	video__fb1 = svga_GM;
+	video__fb2 = vga_mem_page_1;
     }
 }
 
-
 /*
  * XShm code influenced from the DOOM source code.
  * This tries to find some shared memory to use.  It checks for stale segments
@@ -103,7 +89,6 @@ void c_setscreen(int m)
 static void getshm(int size) {
     int			key = ('a'<<24) | ('p'<<16) | ('p'<<8) | 'l';
     struct shmid_ds	shminfo;
-    int			minsize = SCANWIDTH*SCANHEIGHT;
     int			id;
     int			rc;
     int			counter=5;
@@ -113,7 +98,7 @@ static void getshm(int size) {
      */
     do
     {
-	id = shmget((key_t) key, minsize, 0777);
+	id = shmget((key_t) key, 0, 0777);
 	if (id != -1)
 	{
 	    /* we got someone else's ID. check if it's stale. */
@@ -224,7 +209,7 @@ static void getshm(int size) {
     /* attach to the shared memory segment */
     image->data = xshminfo.shmaddr = shmat(id, 0, 0);
 
-    if (!image->data) {
+    if (image->data == -1) {
 	perror("shmat");
 	printf("Could not attach to shared memory\n");
 	exit(1);
@@ -279,42 +264,72 @@ static void c_initialize_colors() {
 	colors[ COLOR_FLASHING_WHITE].green = (255<<8)|255;
 	colors[ COLOR_FLASHING_WHITE].blue = (255<<8)|255;
 
-	/* dhires colors are bass-ackwards because it's easier for me to
-	   deal with the endianness. */
-	colors[0].red = 0;   colors[0].green = 0;   colors[0].blue = 0;
-	colors[1].red = 0;   colors[1].green = 0;   colors[1].blue = 160;/*Magenta <med blue>*/
-	colors[2].red = 0;   colors[2].green = 100; colors[2].blue = 0;/*Brown <dark green>*/
-	colors[3].red = 28;  colors[3].green = 134; colors[3].blue = 255;/*Orange <dodger blue>*/
-	colors[4].red = 165; colors[4].green = 21;  colors[4].blue = 10;/*Dark Green <brown>*/
-	colors[5].red = 165; colors[5].green = 42;  colors[5].blue = 42;/*Gray2 <light gray>*/
-	colors[6].red = 0;   colors[6].green = 255; colors[6].blue = 0;/*Green*/
-	colors[7].red = 160; colors[7].green = 255; colors[7].blue = 160;/*Yellow <Aqua>*/
-	colors[8].red = 112; colors[8].green = 192; colors[8].blue = 140;/*Dark Blue <magenta>*/
-	colors[9].red = 105; colors[9].green = 205; colors[9].blue = 170;/*Purple <Purple>*/
-	colors[0xa].red= 90; colors[0xa].green= 90; colors[0xa].blue = 90;/*Gray1 <dark gray>*/
-	colors[0xb].red=131; colors[0xb].green=111; colors[0xb].blue = 250;/*Pink <slateblue>*/
-	colors[0xc].red=255; colors[0xc].green=198; colors[0xc].blue = 203;/*Med Blue <Orange>*/
-	colors[0xd].red=255; colors[0xd].green=167; colors[0xd].blue = 165;/*Light Blue <Pink>*/
-	colors[0xe].red=255; colors[0xe].green=255; colors[0xe].blue = 0;/*Aqua <Yellow>*/
-	colors[0xf].red=255; colors[0xf].green=255; colors[0xf].blue = 255;/*White*/
+	colors[0x00].red = 0; colors[0x00].green = 0; 
+        colors[0x00].blue = 0;   /* Black */
+	colors[0x10].red = 195; colors[0x10].green = 0;
+        colors[0x10].blue = 48;  /* Magenta */
+	colors[0x20].red = 0; colors[0x20].green = 0;
+        colors[0x20].blue = 130; /* Dark Blue */
+	colors[0x30].red = 166; colors[0x30].green = 52;
+	colors[0x30].blue = 170; /* Purple */
+	colors[0x40].red = 0; colors[0x40].green = 146;
+        colors[0x40].blue = 0;   /* Dark Green */
+	colors[0x50].red = 105; colors[0x50].green = 105;
+        colors[0x50].blue = 105; /* Dark Grey*/
+	colors[0x60].red = 113; colors[0x60].green = 24;
+        colors[0x60].blue = 255; /* Medium Blue */
+	colors[0x70].red = 12; colors[0x70].green = 190;
+	colors[0x70].blue = 235; /* Light Blue */
+	colors[0x80].red = 150; colors[0x80].green = 85;
+        colors[0x80].blue = 40; /* Brown */
+	colors[0x90].red = 255; colors[0xa0].green = 24;
+        colors[0x90].blue = 44; /* Orange */
+	colors[0xa0].red = 150; colors[0xa0].green = 170;
+        colors[0xa0].blue = 170; /* Light Gray */
+	colors[0xb0].red = 255; colors[0xb0].green = 158;
+	colors[0xb0].blue = 150; /* Pink */
+	colors[0xc0].red = 0; colors[0xc0].green = 255;
+        colors[0xc0].blue = 0; /* Green */
+	colors[0xd0].red = 255; colors[0xd0].green = 255;
+	colors[0xd0].blue = 0; /* Yellow */
+	colors[0xe0].red = 130; colors[0xe0].green = 255;
+        colors[0xe0].blue = 130; /* Aqua */
+	colors[0xf0].red = 255; colors[0xf0].green = 255;
+	colors[0xf0].blue = 255; /* White */
 
-	/* lores colors (<<4) */
-	colors[0x00].red=0;	colors[0x00].green=0;   colors[0x00].blue = 0;/*Black*/
-	colors[0x10].red=165;	colors[0x10].green=42;  colors[0x10].blue = 42;/*Magenta*/
-	colors[0x20].red=0;	colors[0x20].green=0;   colors[0x20].blue = 160;/*Dark Blue*/
-	colors[0x30].red=138;	colors[0x30].green=0;   colors[0x30].blue = 138;/*Purple*/
-	colors[0x40].red=0;	colors[0x40].green=100; colors[0x40].blue = 0;/*Dark Green*/
-	colors[0x50].red=128;	colors[0x50].green=128;	colors[0x50].blue = 128;/*Gray*/
-	colors[0x60].red=28;	colors[0x60].green=134; colors[0x60].blue = 255;/*Med Blue*/
-	colors[0x70].red=65;	colors[0x70].green=255; colors[0x70].blue = 255;/*Light Blue*/
-	colors[0x80].red=138;	colors[0x80].green=138; colors[0x80].blue = 0;/*Brown*/
-	colors[0x90].red=255;	colors[0x90].green= 66; colors[0x90].blue = 0;/*Orange*/
-	colors[0xa0].red=90;	colors[0xa0].green= 90;	colors[0xa0].blue = 90;/*Gray2*/
-	colors[0xb0].red=255;	colors[0xb0].green=198; colors[0xb0].blue = 203;/*Pink*/
-	colors[0xc0].red=0;	colors[0xc0].green=255; colors[0xc0].blue = 0;/*Green*/
-	colors[0xd0].red=255;	colors[0xd0].green=255; colors[0xd0].blue = 0;/*Yello*/
-	colors[0xe0].red=160;	colors[0xe0].green=255; colors[0xe0].blue = 160;/*Aqua*/
-	colors[0xf0].red=255;	colors[0xf0].green=255; colors[0xf0].blue = 255;/*White*/
+	/* mirror of lores colors optimized for dhires code */
+	colors[0x00].red = 0; colors[0x00].green = 0; 
+        colors[0x00].blue = 0;   /* Black */
+	colors[0x08].red = 195; colors[0x08].green = 0;
+        colors[0x08].blue = 48;  /* Magenta */
+	colors[0x01].red = 0; colors[0x01].green = 0;
+        colors[0x01].blue = 130; /* Dark Blue */
+	colors[0x09].red = 166; colors[0x09].green = 52;
+	colors[0x09].blue = 170; /* Purple */
+	colors[0x02].red = 0; colors[0x02].green = 146;
+        colors[0x02].blue = 0;   /* Dark Green */
+	colors[0x0a].red = 105; colors[0x0A].green = 105;
+        colors[0x0a].blue = 105; /* Dark Grey*/
+	colors[0x03].red = 113; colors[0x03].green = 24;
+        colors[0x03].blue = 255; /* Medium Blue */
+	colors[0x0b].red = 12; colors[0x0b].green = 190;
+	colors[0x0b].blue = 235; /* Light Blue */
+	colors[0x04].red = 150; colors[0x04].green = 85;
+        colors[0x04].blue = 40; /* Brown */
+	colors[0x0c].red = 255; colors[0x0c].green = 24;
+        colors[0x0c].blue = 44; /* Orange */
+	colors[0x05].red = 150; colors[0x05].green = 170;
+        colors[0x05].blue = 170; /* Light Gray */
+	colors[0x0d].red = 255; colors[0x0d].green = 158;
+	colors[0x0d].blue = 150; /* Pink */
+	colors[0x06].red = 0; colors[0x06].green = 255;
+        colors[0x06].blue = 0; /* Green */
+	colors[0x0e].red = 255; colors[0x0e].green = 255;
+	colors[0x0e].blue = 0; /* Yellow */
+	colors[0x07].red = 130; colors[0x07].green = 255;
+        colors[0x07].blue = 130; /* Aqua */
+	colors[0x0f].red = 255; colors[0x0f].green = 255;
+	colors[0x0f].blue = 255; /* White */
 
 	for (i=0; i<16; i++) {
 	    colors[i].red = (colors[i].red<<8) | colors[i].red;
@@ -335,10 +350,7 @@ static void c_initialize_colors() {
  * Map the X keysyms back into scancodes so the routines in keys.c can deal
  * with it.  We do this to be compatible with what the SVGAlib version does.
  */
-static int keysym_to_scancode(int doRaw) {
-    static char buffer;
-    static KeySym keysym;
-    static XComposeStatus compose;
+static int keysym_to_scancode(void) {
     static int rc = 0xFF;
 
     switch(rc = XKeycodeToKeysym(display, xevent.xkey.keycode, 0))
@@ -364,23 +376,23 @@ static int keysym_to_scancode(int doRaw) {
 	case XK_F10:
 	    rc = 68; break;
 	case XK_Left:
-	    rc = doRaw ? 105 : kLEFT; break;
+	    rc = 105; break;
 	case XK_Right:
-	    rc = doRaw ? 106 : kRIGHT; break;
+	    rc = 106; break;
 	case XK_Down:
-	    rc = doRaw ? 108 : kDOWN; break;
+	    rc = 108; break;
 	case XK_Up:
-	    rc = doRaw ? 103 : kUP; break;
+	    rc = 103; break;
 	case XK_Escape:
-	    rc = doRaw ? 1   : kESC; break;
+	    rc = 1; break;
 	case XK_Return:
-	    rc = doRaw ? 28  : 13; break;
+	    rc = 28; break;
 	case XK_Tab:
-	    rc = doRaw ? 15  : 9; break;
+	    rc = 15; break;
 	case XK_Shift_L:
-	    rc = doRaw ? SCODE_L_SHIFT : -1; break;
+	    rc = SCODE_L_SHIFT; break;
 	case XK_Shift_R:
-	    rc = doRaw ? SCODE_R_SHIFT : -1; break;
+	    rc = SCODE_R_SHIFT; break;
 	case XK_Control_L:
 	    rc = SCODE_L_CTRL; break;
 	case XK_Control_R:
@@ -388,23 +400,31 @@ static int keysym_to_scancode(int doRaw) {
 	case XK_Caps_Lock:
 	    rc = SCODE_CAPS; break;
 	case XK_BackSpace:
-	    rc = doRaw ? 14 : 8; break;
-	case XK_Insert: /* HACK - acts as DEL */
+	    rc = 14; break;
+	case XK_Insert:
 	    rc = 110; break;
 	case XK_Pause:
 	    rc = 119; break;
+	case XK_Break:
+	    /* Pause and Break are the same key, but have different 
+             * scancodes (on PC keyboards).  Ctrl makes the difference.  
+             *
+             * We assume the X server is passing along the distinction to us,
+	     * rather than making us check Ctrl manually.
+             */
+	    rc = 101; break; 
 	case XK_Print:
 	    rc = 99; break;
-	case XK_Delete: /* raw joystick button 0 */
+	case XK_Delete:
 	    rc = 111; break;
-	case XK_End: /* raw joystick button 1 */
-	    rc = doRaw ? 107 : kEND; break;
+	case XK_End:
+	    rc = 107; break;
 	case XK_Home:
-	    rc = kHOME; break;
-	case XK_Page_Down: /* raw joystick button 2 */
-	    rc = doRaw ? 109 : kPGDN; break;
+	    rc = 102; break;
+	case XK_Page_Down:
+	    rc = 109; break;
 	case XK_Page_Up:
-	    rc = kPGUP; break;
+	    rc = 104; break;
 
 	// keypad joystick movement 
 	case XK_KP_5:
@@ -423,13 +443,14 @@ static int keysym_to_scancode(int doRaw) {
 	case XK_KP_Down:
 	    rc = SCODE_J_D; break;
 
+	case XK_Alt_L:
+	    rc = 56; break;
+	case XK_Alt_R:
+	    rc = 100; break;
 
 	default:
-	    if (doRaw && (rc >= XK_space) && (rc <= XK_asciitilde))
+	    if ((rc >= XK_space) && (rc <= XK_asciitilde))
 		rc = xevent.xkey.keycode - 8;
-	    else if (!doRaw && (XLookupString(
-		    (XKeyEvent*)&xevent, &buffer, 1, &keysym, &compose) == 1))
-		rc = (int)keysym;
 	    break;
     }
 
@@ -462,8 +483,35 @@ static void post_image() {
     }
 }
 
-static void c_poll_keyboard_input() {
-    c_keyboard_off();
+static void c_flash_cursor(int on) {
+    // flash only if it's text or mixed modes.
+    if (softswitches & (SS_TEXT|SS_MIXED)) {
+	if (!on) {
+	    colors[ COLOR_FLASHING_BLACK].red = 0;
+	    colors[ COLOR_FLASHING_BLACK].green = 0;
+	    colors[ COLOR_FLASHING_BLACK].blue = 0;
+
+	    colors[ COLOR_FLASHING_WHITE].red = 0xffff;
+	    colors[ COLOR_FLASHING_WHITE].green = 0xffff;
+	    colors[ COLOR_FLASHING_WHITE].blue = 0xffff;
+	} else {
+	    colors[ COLOR_FLASHING_WHITE].red = 0;
+	    colors[ COLOR_FLASHING_WHITE].green = 0;
+	    colors[ COLOR_FLASHING_WHITE].blue = 0;
+
+	    colors[ COLOR_FLASHING_BLACK].red = 0xffff;
+	    colors[ COLOR_FLASHING_BLACK].green = 0xffff;
+	    colors[ COLOR_FLASHING_BLACK].blue = 0xffff;
+	}
+
+	// store the colors to the current colormap
+	XStoreColors(display, cmap, colors, 256);
+    }
+}
+
+/* FIXME: blocking not implemented... */
+void video_sync(int block) {
+    static int flash_count = 0;
     // post the image and loop waiting for it to finish and
     // also process other input events
     post_image();
@@ -479,10 +527,10 @@ LOOP:
 	    goto POLL_FINISHED;
 	switch (xevent.type) {
 	    case KeyPress:
-		c_read_raw_key(keysym_to_scancode(/*raw*/1), 1);
+		c_read_raw_key(keysym_to_scancode(), 1);
 		break;
 	    case KeyRelease:
-		c_read_raw_key(keysym_to_scancode(/*raw*/1), 0);
+		c_read_raw_key(keysym_to_scancode(), 0);
 		break;
 	    default:
 		if (xevent.type == xshmeventtype)
@@ -492,41 +540,19 @@ LOOP:
 	goto LOOP;
 
 POLL_FINISHED:
-    c_periodic_update();
-    signal(SIGVTALRM, c_poll_keyboard_input);/* reinstall */
-    c_keyboard_on();
-}
 
-/* -------------------------------------------------------------------------
-    c_initialize_keyboard()
-   ------------------------------------------------------------------------- */
-
-static void c_initialize_keyboard()
-{
-    signal( SIGVTALRM, c_poll_keyboard_input );
-    c_keyboard_on();
-}
-
-void c_keyboard_on() {
-    static struct itimerval tval, old;
-
-    tval.it_interval.tv_sec = 0L;
-    tval.it_interval.tv_usec = TIMER_DELAY;
-    tval.it_value.tv_sec = 0L;
-    tval.it_value.tv_usec = TIMER_DELAY;
-
-    setitimer( ITIMER_VIRTUAL, &tval, &old );/*set up virtual timer*/
-}
-
-void c_keyboard_off() {
-    static struct itimerval tval, old;
-
-    tval.it_interval.tv_sec = 0L;
-    tval.it_interval.tv_usec = 0L;
-    tval.it_value.tv_sec = 0L;
-    tval.it_value.tv_usec = 0L;
-
-    setitimer( ITIMER_VIRTUAL, &tval, &old );
+    switch (++flash_count)
+    {
+    case 6: 
+        c_flash_cursor(1);
+        break;
+    case 12: 
+        c_flash_cursor(0);
+        flash_count = 0; 
+        break;
+    default:
+        break;
+    }
 }
 
 static Cursor hidecursor() {
@@ -557,7 +583,7 @@ static void parseArgs() {
     }
 }
 
-void c_initialize_video() {
+void video_init() {
 	XSetWindowAttributes attribs;
 	unsigned long	attribmask;
 	int x, y; 	/* window position */
@@ -578,8 +604,8 @@ void c_initialize_video() {
 	progname = argv[0];
 
 	/* give up root privileges. equivalent of vga_init() */
-	seteuid(getuid());
 	setegid(getgid());
+	seteuid(getuid());
 
 	parseArgs();
 
@@ -741,11 +767,6 @@ void c_initialize_video() {
 	    }
 	}
 
-	/* safety */
-	signal(SIGINT, c_shutdown_video);
-	signal(SIGHUP, c_shutdown_video);
-	signal(SIGTERM, c_shutdown_video);
-
 	xshmeventtype = XShmGetEventBase(display) + ShmCompletion;
 
 	/* create the image */
@@ -760,6 +781,11 @@ void c_initialize_video() {
 		    SCANWIDTH,
 		    SCANHEIGHT);
 
+	    if (!image) {
+		fprintf(stderr, "XShmCreateImage failed\n");
+		exit(1);
+	    }
+	
 	    printf("Allocating shared memory %dx%d region\n",
 		    image->bytes_per_line, image->height);
 
@@ -788,18 +814,28 @@ void c_initialize_video() {
 		    SCANHEIGHT,
 		    8/*bitmap_pad*/,
 		    SCANWIDTH/*bytes_per_line*/);
-	}
+	
+	    if (!image) {
+		fprintf(stderr, "XCreateImage failed\n");
+		exit(1);
+	    }
+	
+        }
+
+	svga_GM = video__fb1 = image->data;
+	video__fb2 = vga_mem_page_1;
+
+	memset(video__fb1,0,SCANWIDTH*SCANHEIGHT);
+	memset(video__fb2,0,SCANWIDTH*SCANHEIGHT);
 
-	/* finally, init the keyboard */
-	c_initialize_keyboard();
 }
 
-void c_shutdown_video(void)
+void video_shutdown(void)
 {
     if (doShm) {
 	// Detach from X server
 	if (!XShmDetach(display, &xshminfo))
-	    fprintf(stderr,"XShmDetach() failed in c_shutdown_video()\n");
+	    fprintf(stderr,"XShmDetach() failed in video_shutdown()\n");
 
 	// Release shared memory.
 	shmdt(xshminfo.shmaddr);
@@ -813,116 +849,3 @@ void c_shutdown_video(void)
 
     exit(0);
 }
-
-#define my_pixel(x, y, c) GM[(y)*SCANWIDTH+(x)]=(c)
-void c_clear_video() {
-    int		x,y,i;
-
-    c_setpage( 1 );
-    for (i=0; i<2; i++) {
-	c_setpage(i);
-	for (y = 0; y < SCANHEIGHT; y++)
-	    for (x = 0; x < SCANWIDTH; x++)
-		my_pixel( x, y, 0 );
-    }
-}
-
-/* -------------------------------------------------------------------------
- * c_mygetch():
- * 	get next menu character
- * 	block:	1=block getting input, 0=don't block (this is called by the
- * 		debugger routines when stepping through code to check for a
- * 		user-input break)
- * 	return: -1 (no input) or input character
- * ------------------------------------------------------------------------- */
-
-int c_mygetch(int block) {
-    int c = -1, posts = 0;
-
-    // post the image and wait for it to finish and
-    // also process other input events
-    if (block) {
-	post_image();
-	if (doShm)
-	    ++posts;
-    }
-    do
-    {
-	if (block/* && doShm*/)
-	    XNextEvent(display, &xevent);
-	else if (!XCheckMaskEvent(
-		display,
-		KeyPressMask/*|KeyReleaseMask*/,
-		&xevent))
-	    return c;
-	switch (xevent.type) {
-	    case KeyPress:
-		c = keysym_to_scancode(/*raw*/0);
-		break;
-	    case Expose:
-		post_image();
-		if (doShm)
-		    ++posts;
-		break;
-	    default:
-		if (xevent.type == xshmeventtype) {
-		    --posts;
-		    break;
-		}
-		break;
-	}
-    } while (posts || (block && (c==-1)));
-    return c;
-}
-
-void c_flash_cursor(int on) {
-    // flash only if it's text or mixed modes.
-    if (vmode_text || vmode_mixed) {
-	if (!on) {
-	    colors[ COLOR_FLASHING_BLACK].red = 0;
-	    colors[ COLOR_FLASHING_BLACK].green = 0;
-	    colors[ COLOR_FLASHING_BLACK].blue = 0;
-
-	    colors[ COLOR_FLASHING_WHITE].red = 0xffff;
-	    colors[ COLOR_FLASHING_WHITE].green = 0xffff;
-	    colors[ COLOR_FLASHING_WHITE].blue = 0xffff;
-	} else {
-	    colors[ COLOR_FLASHING_WHITE].red = 0;
-	    colors[ COLOR_FLASHING_WHITE].green = 0;
-	    colors[ COLOR_FLASHING_WHITE].blue = 0;
-
-	    colors[ COLOR_FLASHING_BLACK].red = 0xffff;
-	    colors[ COLOR_FLASHING_BLACK].green = 0xffff;
-	    colors[ COLOR_FLASHING_BLACK].blue = 0xffff;
-	}
-
-	// store the colors to the current colormap
-	XStoreColors(display, cmap, colors, 256);
-    }
-}
-
-void c_video_refresh() {
-    int posts = 0;
-
-    // post the image and wait for it to finish and
-    // also process other input events
-    post_image();
-    if (!doShm)
-	return;
-    ++posts;
-    do
-    {
-	XNextEvent(display, &xevent);
-	switch (xevent.type) {
-	    case Expose:
-		post_image();
-		if (doShm)
-		    ++posts;
-		break;
-	    default:
-		if (xevent.type == xshmeventtype)
-		    --posts;
-		break;
-	}
-    } while (posts);
-}
